@@ -1,11 +1,12 @@
+# More Information: https://developers.google.com/calendar/v3/reference
 import datetime
 
 import pytz
 
-from gcp_pilot.base import GoogleCloudPilotAPI
+from gcp_pilot.base import GoogleCloudPilotAPI, DiscoveryMixin
 
 
-class Calendar(GoogleCloudPilotAPI):
+class Calendar(DiscoveryMixin, GoogleCloudPilotAPI):
     _scopes = ['https://www.googleapis.com/auth/calendar']
 
     def __init__(self, email, timezone: str = 'UTC', **kwargs):
@@ -47,35 +48,27 @@ class Calendar(GoogleCloudPilotAPI):
         min_date = self.date_to_str(starts_at) if starts_at else None
 
         page_size = 100
-        filters = dict(
+        params = dict(
             calendarId=calendar_id,
             timeMin=min_date,
-            maxResults=page_size,
             singleEvents=True,
-            orderBy='startTime',
         )
-        page_token = None
-        while True:
-            filters['pageToken'] = page_token
-            events_result = self.client.events().list(**filters).execute()
 
-            events = events_result.get('items', [])
-            for event in events:
-                yield event
-
-            page_token = events_result.get('nextPageToken')
-            if not page_token:
-                break
+        yield from self._paginate(
+            method=self.client.events().list,
+            result_key='items',
+            params=params,
+            order_by='startTime',
+            limit=page_size,
+        )
 
     def get_calendars(self):
-        page_token = None
-        while True:
-            calendar_list = self.client.calendarList().list(pageToken=page_token).execute()
-            for calendar_list_entry in calendar_list['items']:
-                yield calendar_list_entry
-            page_token = calendar_list.get('nextPageToken')
-            if not page_token:
-                break
+        params = {}
+        yield from self._paginate(
+            method=self.client.calendarList().list,
+            result_key='items',
+            params=params,
+        )
 
     def create_or_update_event(
             self,
@@ -121,19 +114,22 @@ class Calendar(GoogleCloudPilotAPI):
             data['attendees'] = attendees
 
         if event_id:
-            return self.client.events().update(
+            return self._execute(
+                method=self.client.events().update,
                 calendarId=calendar_id,
                 eventId=event_id,
                 body=data,
-            ).execute()
+            )
         else:
-            return self.client.events().insert(
+            return self._execute(
+                method=self.client.events().insert,
                 calendarId=calendar_id,
                 body=data,
-            ).execute()
+            )
 
     def delete_event(self, event_id, calendar_id='primary'):
-        return self.client.events().delete(
+        return self._execute(
+            method=self.client.events().delete,
             calendarId=calendar_id,
             eventId=event_id,
-        ).execute()
+        )
